from rest_framework import serializers

from accounts.business.fields import MemberStatusField
from accounts.models import Member
from accounts.serializers import RelatedUserSerializer, MemberSerializer, \
    RelatedMemberSerializer
from acls.models import Role
from vaultier.business.fields import PermsField
from workspaces.models import Workspace


class InvitationRoleSerializer(serializers.ModelSerializer):

    created_by = serializers.SerializerMethodField('get_created_by')
    to_type = serializers.SerializerMethodField('get_to_type')
    to_name = serializers.SerializerMethodField('get_to_name')

    class Meta:
        model = Role
        fields = ('id', 'to_type', 'to_name', 'created_by', 'created_at',
                  'updated_at')

    def get_to_type(self, obj):
        return obj.type

    def get_to_name(self, obj):
        return obj.get_object().name

    def get_created_by(self, obj):
        return RelatedUserSerializer(instance=obj.created_by).data


class InvitationSerializer(serializers.ModelSerializer):

    roles = serializers.SerializerMethodField('get_roles')
    created_by = RelatedUserSerializer(read_only=True)

    class Meta(MemberSerializer.Meta):
        fields = ('id', 'status', 'invitation_email', 'invitation_hash',
                  'roles', 'created_by', 'created_at', 'updated_at')

    def get_roles(self, obj):
        data = []
        for role in obj.role_set.all():
            data.append(InvitationRoleSerializer(instance=role).data)
        return data


class WorkspaceMembershipSerializer(RelatedMemberSerializer):

    class Meta(RelatedMemberSerializer.Meta):
        fields = ('status', 'id', 'workspace_key')


class WorkspaceSerializer(serializers.ModelSerializer):

    slug = serializers.SlugField(read_only=True)
    created_by = RelatedUserSerializer(read_only=True)
    perms = PermsField()
    membership = serializers.SerializerMethodField('get_membership')

    class Meta:
        model = Workspace
        fields = ('id', 'slug', 'name', 'description', 'membership',
                  'perms', 'created_at', 'updated_at', 'created_by')

    def get_membership(self, obj):
        member = Member.objects.\
            get_concrete_member_to_workspace(obj, self.user)
        if member:
            return WorkspaceMembershipSerializer(member).data
        else:
            return None


class RelatedWorkspaceSerializer(WorkspaceSerializer):

    class Meta(WorkspaceSerializer.Meta):
        fields = ['id', 'slug', 'name']


class WorkspaceKeySerializer(serializers.ModelSerializer):

    public_key = serializers.SerializerMethodField('get_public_key')
    status = serializers.IntegerField(read_only=True)
    workspace_key = serializers.CharField(required=True)
    workspace = RelatedWorkspaceSerializer(read_only=True)
    user = RelatedUserSerializer(read_only=True)

    class Meta:
        model = Member
        fields = ('id', 'public_key', 'workspace_key', 'status',
                  'workspace', 'user', 'created_at', 'updated_at')

    def get_public_key(self, obj):
        return obj.user.public_key

    def save_object(self, obj, **kwargs):
        obj.status = MemberStatusField.STATUS_MEMBER
        return super(WorkspaceKeySerializer, self).save_object(obj, **kwargs)


class ShortenedWorkspaceKeySerializer(serializers.ModelSerializer):

    class Meta:
        model = Member
        fields = ('id', 'workspace', 'status', 'user', 'created_at',
                  'updated_at')



from django.db.models.loading import get_model
from cards.models import Card
from libs.version.condition import RequiredFieldEventCondition
from libs.changes.changes import SOFT_DELETE, INSERT, UPDATE
from libs.version.manipulator import register_manipulator_signal, \
    ModelCreatedManipulator, ModelUpdatedManipulator, \
    ModelSoftDeletedManipulator, register_manipulator_class, \
    ModelMovedManipulator


def register_signals():
    register_manipulator_class('card_created_manipulator',
                               ModelCreatedManipulator)
    register_manipulator_class('card_updated_manipulator',
                               ModelUpdatedManipulator)
    register_manipulator_class('card_deleted_manipulator',
                               ModelSoftDeletedManipulator)
    register_manipulator_class('card_moved_manipulator',
                               ModelMovedManipulator)

    version_model = get_model('versions', 'Version')

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='card_deleted_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Card,
            required_fields=['deleted_at'],
            required_event=SOFT_DELETE,
        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='card_updated_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Card,
            required_fields=['name', 'description'],
            required_event=UPDATE,
        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='card_created_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Card,
            required_fields=None,
            required_event=INSERT,
        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='card_moved_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Card,
            required_fields=['vault_id'],
            required_event=UPDATE,
        )
    )



from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^libs\.lowercasefield\.lowercasefield\."
                             "LowerCaseCharField"])
add_introspection_rules([], ["^secrets\.business\.fields\.SecretTypeField"])



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_201_CREATED, HTTP_200_OK, \
    HTTP_204_NO_CONTENT
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call, retrieve_card_api_call, \
    list_cards_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from vaultier.test.tools.workspace.api import create_workspace_api_call
from vaults.tests.api import create_vault_api_call, delete_vault_api_call


class ApiCardTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_create_card(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        response = create_card_api_call(user1token, name="card_in_vault",
                                        vault=vault.get('id'))

        self.assertEqual(
            response.status_code,
            HTTP_201_CREATED,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_020_delete_card(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #delete vault
        response = delete_vault_api_call(user1token, card.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

    def test_030_list_cards(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #list cards
        response = list_cards_api_call(user1token, vault=vault.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

    def test_040_retrieve_card(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #retrieve card
        response = retrieve_card_api_call(user1token, card.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )


def card_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiCardTest))
    return suite



from django.db.models.loading import get_model
from django.db.models.manager import Manager
from django.db.models.query_utils import Q
from accounts.business.fields import MemberStatusField
from acls.business.fields import AclLevelField
from libs.changes.changes import INSERT
from vaults.models import Vault
from acls.business.mailer import GrantedAccessMailer


class RoleManager(Manager):

    def on_model(self, signal=None, sender=None, instance=None,
                 event_type=None, **kwargs):
        if event_type == INSERT and \
                (instance.member.status == MemberStatusField.STATUS_MEMBER or
                 instance.member.status ==
                 MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY):
            self.send_granted_access(instance)

    def send_granted_access(self, instance):
        sender = GrantedAccessMailer(object=instance.get_object())
        sender.send()

    def all_for_member(self, member_id):
        member = get_model('workspaces', 'Member').objects.get(pk=member_id)
        return self.all_for_user(member.user)

    def all_for_user(self, user):
        # all workspaces user has permission write
        workspaces = get_model('workspaces', 'Workspace').objects.filter(
            acl__level=AclLevelField.LEVEL_READ,
            acl__user=user
        ).distinct()

        # all vaults user has permission read
        vaults = Vault.objects.filter(
            acl__level=AclLevelField.LEVEL_READ,
            acl__user=user
        ).distinct()

        # all cards user has permission read
        cards = get_model('cards', 'Card').objects.filter(
            acl__level=AclLevelField.LEVEL_READ,
            acl__user=user
        ).distinct()

        # all roles (to_workspace, to_vault, to_card) related to workspaces
        # in which user has permission to write
        roles = self.filter(
            Q(to_workspace__in=workspaces) | Q(to_vault__in=vaults) |
            Q(to_card__in=cards)
        ).distinct()

        return roles

    def create_or_update_role(self, role):
        existing = None
        try:
            existing = get_model('acls', 'Role').objects.filter(
                member=role.member,
                to_workspace=role.to_workspace,
                to_vault=role.to_vault,
                to_card=role.to_card
            )[0]
        except:
            pass

        if not existing:
            role.save()
            return role
        else:
            existing.level = role.level
            existing.member = role.member
            existing.save()
            return existing




class Manager(object):
    _user = None
    _enabled = True
    _user_required = True

    def set_user_required(self, user_required):
        self._user_required = user_required

    def get_user_required(self):
        return self._user_required

    def set_user(self, user):
        self._user = user

    def get_user(self):
        if self._user_required and (not self._user or
                                    self._user.is_anonymous()):
            msg = 'To store version valid user is required on ' \
                  'version_context_manager'
            raise Exception(msg)
        return self._user

    def set_enabled(self, enabled):
        self._enabled = enabled

    def get_enabled(self):
        return self._enabled


version_context_manager = Manager()


class VersionContextAwareApiViewMixin(object):
    def initialize_request(self, request, *args, **kargs):
        request = super(VersionContextAwareApiViewMixin, self) \
            .initialize_request(request, *args, **kargs)
        version_context_manager.set_user(request.user)

        return request



from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^vaultier\.business\.fields\."
                             "ObjectReferenceTypeField"])
add_introspection_rules([], ["^acls\.business\.fields\.RoleLevelField"])
add_introspection_rules([], ["^libs\.lowercasefield\.lowercasefield\."
                             "LowerCaseCharField"])
add_introspection_rules([], ["^acls\.business\.fields\.AclDirectionField"])
add_introspection_rules([], ["^accounts\.business\.fields\.MemberStatusField"])



from rest_framework import serializers
from accounts.serializers import RelatedUserSerializer
from vaultier.business.fields import PermsField
from vaults.models import Vault


class VaultSerializer(serializers.ModelSerializer):
    slug = serializers.SlugField(read_only=True)
    created_by = RelatedUserSerializer(read_only=True)
    perms = PermsField()

    def restore_fields(self, data, files):
        if self.context.get('view').action != 'create':
            self.fields.get('workspace').read_only = True
        return super(VaultSerializer, self).restore_fields(data, files)

    class Meta:
        model = Vault
        fields = ('id', 'slug', 'name', 'color', 'description', 'workspace',
                  'perms', 'created_at', 'updated_at', 'created_by')


class RelatedVaultSerializer(VaultSerializer):

    class Meta(VaultSerializer.Meta):
        fields = ['id', 'slug', 'name', 'color']



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_200_OK
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call
from libs.version.context import version_context_manager
from .api import search_api_call
from vaultier.test.tools import format_response
from vaults.tests.api import create_vault_api_call
from workspaces.tests.api import create_workspace_api_call


class ApiSearchTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_search_vault_case_insensitive(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="this is vault",
                                      workspace=workspace.get('id')).data

        response = search_api_call(user1token, query='VaUlT')

        # should be ok
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # one vault should be returned
        self.assertEqual(
            len(response.data.get('vaults')),
            1,
            format_response(response)
        )

    def test_020_search_vault_permissions(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace1 = create_workspace_api_call(user1token,
                                               name='Workspace').data

        # create vault
        vault1 = create_vault_api_call(user1token, name="this is vault 1",
                                       workspace=workspace1.get('id')).data

        # create workspace
        workspace2 = create_workspace_api_call(user2token,
                                               name='Workspace').data

        #create vault
        vault2 = create_vault_api_call(user2token, name="this is vault 2",
                                       workspace=workspace2.get('id')).data

        response = search_api_call(user1token, query='VaUlT')
        # one vault should be returned
        self.assertEqual(
            response.data.get('vaults')[0].get('name'),
            'this is vault 1',
            format_response(response)
        )

        response = search_api_call(user2token, query='VaUlT')
        # one vault should be returned
        self.assertEqual(
            response.data.get('vaults')[0].get('name'),
            'this is vault 2',
            format_response(response)
        )

    def test_030_search_card_case_insensitive(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="this is vault",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="this is card",
                                    vault=vault.get('id')).data

        response = search_api_call(user1token, query='ThIs')

        # should be ok
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # one vault should be returned
        self.assertEqual(
            len(response.data.get('vaults')),
            1,
            format_response(response)
        )

        # one card should be returned
        self.assertEqual(
            len(response.data.get('cards')),
            1,
            format_response(response)
        )

    def test_040_search_card_permissions(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace1 = create_workspace_api_call(user1token,
                                               name='Workspace').data

        # create vault
        vault1 = create_vault_api_call(user1token, name="this is vault 1",
                                       workspace=workspace1.get('id')).data

        #create card
        card1 = create_card_api_call(user1token, name="this is card 1",
                                     vault=vault1.get('id')).data

        # create workspace
        workspace2 = create_workspace_api_call(user2token,
                                               name='Workspace').data

        #create vault
        vault2 = create_vault_api_call(user2token, name="this is vault 2",
                                       workspace=workspace2.get('id')).data

        #create card
        card2 = create_card_api_call(user2token, name="this is card 2",
                                     vault=vault2.get('id')).data

        response = search_api_call(user1token, query='card')
        # one vault should be returned
        self.assertEqual(
            response.data.get('cards')[0].get('name'),
            'this is card 1',
            format_response(response)
        )

        response = search_api_call(user2token, query='card')
        # one vault should be returned
        self.assertEqual(
            response.data.get('cards')[0].get('name'),
            'this is card 2',
            format_response(response)
        )


def search_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiSearchTest))
    return suite



from __future__ import absolute_import
from celery import shared_task
from .models import Member
from .models import Token


@shared_task()
def task_garbage_collector():
    """
    Removes expired tokens from DB
    """
    Token.objects.clean_old_tokens()
    Member.objects.clean_old_invitations()



from django.db.models.manager import Manager
from django.db.models import Q
from acls.business.fields import AclLevelField
from libs.softdelete.softdelete import SoftDeleteManagerMixin


class CardManager(SoftDeleteManagerMixin, Manager):

    def get_queryset(self):
        queryset = super(CardManager, self).get_queryset()
        return queryset.filter(
            vault__workspace__deleted_at=None,
            vault__deleted_at=None,
        )

    def search(self, user, query, max_results=5):
        list = query.split()
        result = self.all_for_user(user).filter(
            Q(reduce(lambda x, y: x | y,
                     [Q(name__icontains=word) for word in list])) |
            Q(reduce(lambda x, y: x | y,
                     [Q(description__icontains=word) for word in list])) |
            Q(reduce(lambda x, y: x | y,
                     [Q(secret__name__icontains=word) for word in list]))
        ).order_by('updated_at')
        return result[:max_results]

    def all_for_user(self, user):
        cards = self.filter(
            acl__user=user,
            acl__level=AclLevelField.LEVEL_READ
        ).distinct()

        return cards



from accounts.models import Member
from cards.models import Card
from acls.models import Role
from secrets.models import Secret
from vaults.models import Vault
from acls.business.perms.materializer import CreateRoleMaterializer, \
    UpdateRoleLevelMaterializer, UpdateRoleMemberMaterializer, \
    UpdateMemberUserMaterializer, InsertedObjectMaterializer, \
    MovedObjectMaterializer
from libs.changes.changes import post_change, INSERT, UPDATE


# when role is created and member has related user
def on_role_created(signal=None, sender=None, instance=None, event_type=None,
                    **kwargs):
    if event_type == INSERT and instance.member.user:
        materializer = CreateRoleMaterializer(instance)
        materializer.materialize(instance.get_object())


# when role level is updated
def on_role_level_updated(signal=None, sender=None, instance=None,
                          event_type=None, overwritten_values=None, **kwargs):
    if event_type == UPDATE and 'level' in overwritten_values:
        materializer = UpdateRoleLevelMaterializer(instance)
        materializer.materialize()


# when member of role is updated
def on_role_member_updated(signal=None, sender=None, instance=None,
                           event_type=None, overwritten_values=None, **kwargs):
    if event_type == UPDATE and 'member' in overwritten_values and \
            instance.member.user:
        materializer = UpdateRoleMemberMaterializer(instance)
        materializer.materialize()


# when invited member becames regular member
def on_member_is_regular(signal=None, sender=None, instance=None,
                         event_type=None, overwritten_values=None, **kwargs):
    if event_type == UPDATE and 'user' in overwritten_values and \
            instance.user:
        materializer = UpdateMemberUserMaterializer(instance)
        materializer.materialize()


# when new object is inserted parent acls are inherited
def on_object_inserted(signal=None, sender=None, instance=None,
                       event_type=None, **kwargs):
    if event_type == INSERT:
        materializer = InsertedObjectMaterializer(instance)

        # materialize CREATE roles
        materializer.materialize_role()

        # materialize acls
        materializer.materialize()


# when object is moved (parent id is changed)
def on_object_moved(signal=None, sender=None, instance=None, event_type=None,
                    overwritten_values=None, **kwargs):
    if event_type == UPDATE:
        materializer = None

        if instance.__class__.__name__ == 'Card' and \
                'vault' in overwritten_values:
            materializer = MovedObjectMaterializer(instance)
        if instance.__class__.__name__ == 'Secret' and \
                'card' in overwritten_values:
            materializer = MovedObjectMaterializer(instance.card)

        if materializer:
            materializer.materialize()


def register_signals():
    post_change.connect(on_role_created, sender=Role)
    post_change.connect(on_role_level_updated, sender=Role)
    post_change.connect(on_role_member_updated, sender=Role)
    post_change.connect(on_member_is_regular, sender=Member)

    post_change.connect(on_object_inserted, sender=Vault)
    post_change.connect(on_object_inserted, sender=Card)

    post_change.connect(on_object_moved, sender=Card)
    post_change.connect(on_object_moved, sender=Secret)



from .secret import secret_suite
from .secret_blob import secret_blob_suite
from .secret_blob_version import secret_blob_version_suite
from .secret_perms import secret_perms_suite
from .secret_softdelete import secret_softdelete_suite
from .secret_version import secret_version_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    s.addTest(secret_suite())
    s.addTest(secret_blob_suite())
    s.addTest(secret_blob_version_suite())
    s.addTest(secret_perms_suite())
    s.addTest(secret_softdelete_suite())
    s.addTest(secret_version_suite())
    return s



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'User'
        db.create_table(u'vaultier_user', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('password', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('last_login', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('nickname', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('public_key', self.gf('django.db.models.fields.CharField')(max_length=1024)),
            ('email', self.gf('libs.lowercasefield.lowercasefield.LowerCaseCharField')(unique=True, max_length=255)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal(u'accounts', ['User'])

        # Adding model 'Token'
        db.create_table(u'vaultier_token', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('token', self.gf('django.db.models.fields.CharField')(unique=True, max_length=64, db_index=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['accounts.User'])),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('last_used_at', self.gf('django.db.models.fields.DateTimeField')(null=True)),
        ))
        db.send_create_signal(u'accounts', ['Token'])

        # Adding model 'LostKey'
        db.create_table(u'vaultier_lost_key', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('hash', self.gf('django.db.models.fields.TextField')()),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='distracted', on_delete=models.PROTECT, to=orm['accounts.User'])),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('expires_at', self.gf('django.db.models.fields.DateTimeField')()),
            ('used', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'accounts', ['LostKey'])

        # Adding model 'Member'
        db.create_table(u'vaultier_member', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='membership', null=True, to=orm['accounts.User'])),
            ('invitation_hash', self.gf('django.db.models.fields.CharField')(max_length=64, unique=True, null=True)),
            ('invitation_email', self.gf('libs.lowercasefield.lowercasefield.LowerCaseCharField')(max_length=1024, null=True)),
            ('workspace_key', self.gf('django.db.models.fields.CharField')(max_length=4096)),
            ('status', self.gf('accounts.business.fields.MemberStatusField')(default=3)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='members_created', on_delete=models.PROTECT, to=orm['accounts.User'])),
        ))
        db.send_create_signal(u'accounts', ['Member'])


    def backwards(self, orm):
        # Deleting model 'User'
        db.delete_table(u'vaultier_user')

        # Deleting model 'Token'
        db.delete_table(u'vaultier_token')

        # Deleting model 'LostKey'
        db.delete_table(u'vaultier_lost_key')

        # Deleting model 'Member'
        db.delete_table(u'vaultier_member')


    models = {
        u'accounts.lostkey': {
            'Meta': {'object_name': 'LostKey', 'db_table': "u'vaultier_lost_key'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'distracted'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            'expires_at': ('django.db.models.fields.DateTimeField', [], {}),
            'hash': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'used': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        u'accounts.member': {
            'Meta': {'object_name': 'Member', 'db_table': "u'vaultier_member'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'members_created'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invitation_email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'max_length': '1024', 'null': 'True'}),
            'invitation_hash': ('django.db.models.fields.CharField', [], {'max_length': '64', 'unique': 'True', 'null': 'True'}),
            'status': ('accounts.business.fields.MemberStatusField', [], {'default': '3'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'membership'", 'null': 'True', 'to': u"orm['accounts.User']"}),
            'workspace_key': ('django.db.models.fields.CharField', [], {'max_length': '4096'})
        },
        u'accounts.token': {
            'Meta': {'object_name': 'Token', 'db_table': "u'vaultier_token'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_used_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'token': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64', 'db_index': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']"})
        },
        u'accounts.user': {
            'Meta': {'object_name': 'User', 'db_table': "u'vaultier_user'"},
            'email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'nickname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'public_key': ('django.db.models.fields.CharField', [], {'max_length': '1024'})
        }
    }

    complete_apps = ['accounts']


# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name="Vaultier",
    install_requires=[
        'BeautifulSoup==3.2.1',
        'Django==1.6',
        'South==0.8.4',
        'argparse==1.2.1',
        'cssutils==1.0',
        'django-appconf==0.6',
        'django-extensions==1.2.5',
        'django-filter==0.7',
        'djangorestframework==2.3.12',
        'flup==1.0.2',
        'html2text==3.200.3',
        'jsonfield==0.9.20',
        'psycopg2==2.5.1',
        'pycrypto==2.6.1',
        'pynliner==0.5.0',
        'six==1.4.1',
        'wsgiref==0.1.2',
        'raven==5.0.0',
        'pytz==2014.7',
        'celery==3.1.9',
        'requests==2.3.0',
        'python-dateutil==2.2',
        'logan==0.5.10',
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Information Technology",
        "Topic :: Security",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],
    entry_points={
        'console_scripts': [
            'vaultier = vaultier.runner:main'
        ],
    },
    data_files = [
        (
            'vaultier-config-examples',
            ['cfg/nginx', 'cfg/supervisord', 'cfg/uwsgi']
        )
    ],
    packages=['vaultier', ],
    author=u"rclick s.r.o.",
    author_email='info@rclick.cz',
    zip_safe=True,
    include_package_data = True,
    description="Vaultier project backend",
    long_description="Vaultier combines strong security with the option to "
                     "share sensitive data within the working teams, supliers "
                     "and clients. Concept of Vaultier tool was based on our "
                     "own needs and was designed to solve our problem to "
                     "collaborate on hundreds of login credentials with the "
                     "proper level of security and access control.",
    version="0.7.0",
    url="http://vaultier.org"
)


import urlparse
from vaultier.business.mailer import VaultierMailer
from django.conf import settings


class InvitationMailer(VaultierMailer):
    """
    Sends email whenever user is send to new space

    """
    _template = 'mailer/invitation'
    subject = '[Vaultier] You have been invited'

    def send(self, **kwargs):
        """
        Adds recipient from object

        :param kwargs:
        :return:
        """
        if self.object:
            self.add_to(self.object.invitation_email)
        return super(InvitationMailer, self).send(**kwargs)

    def _build_context(self, **kwargs):
        """
        Adds extra variables to template Context

        :param kwargs: dict
        :return: Context
        """
        kwargs.update({
            'SITE_URL': settings.SITE_URL,
            'url': self._build_url()
        })
        return super(InvitationMailer, self)._build_context(**kwargs)

    def _build_url(self):
        """
        Generate frontend url to granted object

        :return: str
        """
        return urlparse.urljoin(
            settings.SITE_URL, '#/invitations/use/{}/{}/'.format(
                self.object.id, self.object.invitation_hash
            )
        )


class WorkspaceKeyTransferMailer(VaultierMailer):

    _template = 'mailer/transfer_workspace_key'

    def format_subject(self):
        """
        Return mail formatted subject

        :return:
        """
        return '[Vaultier] You can access the workspace: {}'.format(
            self.object.workspace.name
        )

    def send(self, **kwargs):
        """
        Adds recipient from object

        :param kwargs:
        :return:
        """
        if self.object:
            self.add_to(self.object.user.email)
        return super(WorkspaceKeyTransferMailer, self).send(**kwargs)

    def _build_context(self, **kwargs):
        """
        Adds extra variables to template Context

        :param kwargs: dict
        :return: Context
        """
        kwargs.update({
            'granted_workspace': self.object.workspace.name,
            'url': self._build_url()
        })
        return super(WorkspaceKeyTransferMailer, self)._build_context(**kwargs)

    def _build_url(self):
        """
        Generate frontend url to granted object

        :return: str
        """
        return urlparse.urljoin(settings.SITE_URL, '#/workspaces/w/{}/'.format(
            self.object.workspace.slug))



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call
from libs.version.context import version_context_manager
from libs.version.manipulator import ACTION_CREATED, ACTION_UPDATED, \
    ACTION_SOFTDELETED, ACTION_MOVED
from secrets.business.fields import SecretTypeField
from secrets.tests.api import create_secret_api_call, update_secret_api_call, \
    delete_secret_api_call
from vaultier.test.tools import AssertionsMixin
from vaults.tests.api import create_vault_api_call
from versions.models import Version
from workspaces.tests.api import create_workspace_api_call


class SecretVersionTest(AssertionsMixin, TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(True)

    def create_secret(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        # create vault
        vault = create_vault_api_call(
            user1token, workspace=workspace.get('id'), name='vault').data

        #create card
        card = create_card_api_call(
            user1token, name="card_in_vault", vault=vault.get('id')).data

        secret = create_secret_api_call(
            user1token, type=SecretTypeField.SECRET_TYPE_PASSWORD,
            name='secret_in_card', card=card.get('id'), data="mocked_data"
        ).data

        return user1, user1token, workspace, vault, card, secret

    @unittest.skip("should be fixed asap")
    def test_secret_010_create(self):
        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id')
        )

        # one version should be there
        self.assertEquals(versions.count(), 1)

        # compare version
        self.assert_model(versions[0], {
            'versioned_id': card.get('id'),
            'versioned_related_id': card.get('id'),
            'action_id': ACTION_CREATED
        })

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id, card.get('id'))

        # assert user has been stored with version
        self.assertEqual(versions[0].created_by.id, user1.get('id'))

        # compare revision data
        self.assert_dict(versions[0].revert_data, {})

    @unittest.skip("should be fixed asap")
    def test_secret_020_update(self):
        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        #update secret
        secret = update_secret_api_call(
            user1token, secret.get('id'), name='renamed_secret',
            data='updated_mocked_data').data

        #get versions
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            action_id=ACTION_UPDATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {
            'name': 'secret_in_card',
            'data': 'mocked_data'
        })

        #update secret again
        secret = update_secret_api_call(
            user1token, secret.get('id'), name='renamed_secret_again',
            data='updated_mocked_data_again').data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            action_id=ACTION_UPDATED
        )

        # two versions should be there
        self.assertEquals(versions.count(), 2)

        # assert related id
        self.assertEqual(versions[1].versioned_related_type.model, 'card')
        self.assertEqual(versions[1].versioned_related_id, card.get('id'))

        # compare revision data
        self.assert_dict(versions[1].revert_data, {
            'name': 'renamed_secret',
            'data': 'updated_mocked_data'
        })

    def test_secret_030_delete(self):
        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            action_id=ACTION_CREATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        delete_secret_api_call(user1token, secret.get('id'))

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            action_id=ACTION_SOFTDELETED
        )

        # one versions should be there, created and deleted
        self.assertEquals(versions.count(), 1)

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id, card.get('id'))

        # compare revision data
        self.assert_dict(versions[0].revert_data, {
            'deleted_at': None
        })

    def test_secret_040_move(self):
        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        # create second vault
        second_card = create_card_api_call(
            user1token, vault=vault.get('id'), name='second_card').data

        secret = update_secret_api_call(
            user1token, secret.get('id'), card=second_card.get('id'),
            name='renamed_secret').data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id')
        )

        # 3 versions should be there, created, update and moved
        self.assertEquals(versions.count(), 3)

        #get versions
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            action_id=ACTION_MOVED
        )

        # compare revision data
        self.assert_dict(versions[0].revert_data, {'card_id': card.get('id')})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id,
                         second_card.get('id'))

    def test_secret_050_move_with_other_versions(self):
        # Because secrets version are always related to cards
        # when secret is moved we have to ensure that card
        # was modified for all other secret versions

        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        # create second vault
        second_card = create_card_api_call(
            user1token, vault=vault.get('id'), name='second_card').data

        secret = update_secret_api_call(
            user1token, secret.get('id'), card=second_card.get('id'),
            name='renamed_secret',).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
        )

        # 3 versions should be there, created, update and moved
        self.assertEquals(versions.count(), 3)

        # we have to assert e.g create version card has been also changed
        for version in versions:
            self.assertEqual(
                version.versioned_related_id, second_card.get('id'))
            self.assertEqual(version.versioned_related_type.model, 'card')


def secret_version_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        SecretVersionTest))
    return suite



from acls.business.fields import AclLevelField, AclDirectionField
from ...models import Acl


class ReadAclStrategy(object):
    def construct_acl(self, role, object):
        acl = Acl()
        acl.set_object(object)
        acl.role = role
        acl.user = role.member.user
        return acl

    def acl_for_child(self, role, object):
        acls = []

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_DOWN
        acl.level = AclLevelField.LEVEL_READ
        acls.append(acl)

        return acls

    def acl_for_parent(self, role, object):
        acls = []

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_UP
        acl.level = AclLevelField.LEVEL_READ
        acls.append(acl)

        return acls

    def acl_for_object(self, role, object):
        acls = []

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_CURRENT
        acl.level = AclLevelField.LEVEL_READ
        acls.append(acl)

        return acls


class WriteAclStrategy(ReadAclStrategy):
    def acl_for_child(self, role, object):
        acls = super(WriteAclStrategy, self).acl_for_child(role, object)

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_DOWN
        acl.level = AclLevelField.LEVEL_WRITE
        acls.append(acl)

        return acls

    def acl_for_object(self, role, object):
        acls = super(WriteAclStrategy, self).acl_for_object(role, object)

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_CURRENT
        acl.level = AclLevelField.LEVEL_WRITE
        acls.append(acl)

        return acls


class CreateAclStrategy(ReadAclStrategy):
    def acl_for_child(self, role, object):
        return []

    def acl_for_object(self, role, object):
        acls = super(CreateAclStrategy, self).acl_for_object(role, object)

        acl = self.construct_acl(role, object)
        acl.direction = AclDirectionField.DIR_CURRENT
        acl.level = AclLevelField.LEVEL_CREATE
        acls.append(acl)

        return acls



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call, delete_card_api_call
from libs.version.context import version_context_manager
from secrets.business.fields import SecretTypeField
from secrets.models import Secret
from secrets.tests.api import create_secret_api_call, delete_secret_api_call
from vaults.tests.api import create_vault_api_call, delete_vault_api_call
from workspaces.tests.api import create_workspace_api_call, \
    delete_workspace_api_call


class SecretSoftDeleteTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def create_secret(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        #create vault
        vault = create_vault_api_call(
            user1token, name="vault_in_workspace",
            workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(
            user1token, name="card_in_vault", vault=vault.get('id')).data

        #create secret
        secret = create_secret_api_call(
            user1token, name="secret_in_card", card=card.get('id'),
            type=SecretTypeField.SECRET_TYPE_PASSWORD
        ).data

        return user1token, workspace, vault, card, secret

    def test_010_softdelete(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        delete_secret_api_call(user1token, secret.get('id'))

        secrets = Secret.objects.filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 0)

        secrets = Secret.objects.include_deleted().filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 1)

    def test_020_softdelete_card(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        delete_card_api_call(user1token, card.get('id'))

        secrets = Secret.objects.filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 0)

        secrets = Secret.objects.include_deleted().filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 1)

    def test_030_softdelete_vault(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        delete_vault_api_call(user1token, vault.get('id'))

        secrets = Secret.objects.filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 0)

        secrets = Secret.objects.include_deleted().filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 1)

    @unittest.skip("should be fixed asap")
    def test_040_softdelete_workspace(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        delete_workspace_api_call(user1token, vault.get('id'))

        secrets = Secret.objects.filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 0)

        secrets = Secret.objects.include_deleted().filter(id=secret.get('id'))
        self.assertEquals(secrets.count(), 1)


def secret_softdelete_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        SecretSoftDeleteTest))
    return suite



import urlparse
from vaultier.business.mailer import VaultierMailer
from django.conf import settings


class GrantedAccessMailer(VaultierMailer):
    """
    Sends an email when the user has granted access to a any
    workspace,vault ot card
    """

    _template = 'mailer/granted_access'

    def format_subject(self):
        """
        Return mail subject

        :return:
        """
        return '[Vaultier] You have been granted access to {}'.format(
            self.object.__class__.__name__.lower()
        )

    def send(self, **kwargs):
        """
        Get the right email to which the email will be send

        :return: list
        """
        if hasattr(self.object, 'member'):
            self.add_to(self.object.member.user.email)
        elif hasattr(self.object, 'membership'):
            self.add_to(self.object.created_by.email)
        return super(GrantedAccessMailer, self).send(**kwargs)

    def _build_context(self, **kwargs):
        """
        Adds extra variables to template Context

        :return: Context
        """

        kwargs.update({
            'type': self.object.__class__.__name__.lower(),
            'SITE_URL': settings.SITE_URL,
            'name': getattr(self.object, 'name'),
            'url': self._build_url()
        })
        return super(GrantedAccessMailer, self)._build_context(**kwargs)

    def _build_url(self):
        """
        Generate frontend url to granted object

        :return: str
        """
        u = urlparse.urljoin(settings.SITE_URL, '/#/')

        m = self.object.__class__.__name__

        if m == 'Workspace':
            return urlparse.urljoin(
                u, 'workspaces/w/{}'.format(self.object.slug)
            ),
        elif m == 'Vault':
            return urlparse.urljoin(
                u, 'workspaces/w/{}/vaults/v/{}'.format(
                    self.object.workspace.slug, self.object.slug)
            ),
        elif m == 'Card':
            return urlparse.urljoin(
                u, '/workspaces/w/{}/vaults/v/{}/cards/c/{}'.format(
                    self.object.vault.workspace.slug, self.object.vault.slug,
                    self.object.slug)
            ),

        return None



from django.http.response import HttpResponse
from django.shortcuts import render, redirect
from django.template.context import Context
from django.conf import settings
import json
import pkg_resources
from rest_framework.views import APIView
from accounts.models import User


def index(request):

    return render(request, 'index.html', Context({
        'dev_shared_key': settings.VAULTIER.get('dev_shared_key'),
    }))


class ConfigView(APIView):
    """
    View to provide JS configuration
    """
    def get(self, request):
        """
        Get configuration from settings, format it and return
        """
        # get settings and transform it to json
        conf_settings = json.dumps({
            'VERSION': pkg_resources.get_distribution("Vaultier").version,
            'raven_key': settings.VAULTIER.get('raven_key'),
            'invitation_lifetime': settings.VAULTIER.get(
                'invitation_lifetime'),
            'registration_allow': settings.VAULTIER.get('registration_allow'),
            'registration_enforce': not bool(User.objects.all().count()),
            # dev
            'dev_shared_key': settings.VAULTIER.get('dev_shared_key'),
            'dev_show_token': settings.VAULTIER.get('dev_show_token'),
            'dev_email': settings.VAULTIER.get('dev_email')
        })
        # add settings to script
        script = 'InitializeConfig = function(app) { ' \
                 'app.Config = Ember.Object.extend(%s); }' % conf_settings

        return HttpResponse(script, content_type='text/javascript')


def error404(request):
    return redirect('/#'+request.path)

# def dev_mail(request):
#    context = build_context(Member.objects.filter(
#        status=MemberStatusField.STATUS_INVITED).reverse()[0])
#    plain, html = render_email('mailer/invitation', context)
#    return HttpResponse(html)



from time import time
from django.core.urlresolvers import reverse
from rest_framework.test import APIClient
# todo: Backend?
# from accounts.business.authentication import Backend
from accounts.business.fields import RecoverTypeField
from vaultier.test.tools import FileAccessMixin, VaultierAPIClient
from django.utils import timezone


def auth_api_call(email=None, date=None, signature=None):
    url = reverse('auth-auth')
    client = APIClient()
    m = FileAccessMixin()

    if not date:
        date = timezone.now()

    if not signature:
        privkey = m.read_file('vaultier.key')
        # todo: Backend?
        # signature = Backend.sign(privkey, email, date)
        signature = None

    response = client.post(url, {'email': email,
                                 'date': date,
                                 'signature': signature}
                           )
    return response


def register_api_call(*args, **kwargs):
    m = FileAccessMixin()
    pubkey = m.read_file('vaultier.pub')
    kwargs['public_key'] = pubkey

    url = reverse('user-list')
    client = APIClient()
    kwargs.update({'timestamp': int(time())})
    response = client.post(url, kwargs)
    return response


def invite_member_api_call(token, email=None, workspace=None, send=True,
                           resend=True):
    url = reverse('member-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, {
        'email': email,
        'workspace': workspace,
        'send': send,
        'resend': resend
    })
    return response


def list_members_api_call(token, workspace):
    url = reverse('member-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url, {'workspace': workspace})
    return response


def delete_member_api_call(token, member):
    url = reverse('member-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def create_lost_keys_api_call(email, **kwargs):
    """
    Call to lost_key create view
    :param email: user email
    :param kwargs:
    :return: Response
    """
    client = VaultierAPIClient()
    kwargs['email'] = email
    url = reverse('lost_keys-list')
    response = client.post(url, data={'email': email}, kwargs=kwargs)
    return response


def update_lost_key_api_rebuild_call(lost_key_id, auth_hash=None,
                                     public_key=None):
    """
    Call to update view with parameter recover_type set to
    RecoverTypeField.REBUILD
    :param lost_key_id: int
    :param auth_hash: str
    :param public_key: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = "{}?hash={}".format(reverse('lost_keys-detail', args=(lost_key_id,)),
                              auth_hash)
    return client.put(url, data={
        'public_key': public_key, 'recover_type': RecoverTypeField.REBUILD})


def update_lost_key_api_disable_call(lost_key_id, auth_hash=None,
                                     public_key=None):
    """
    Call to update view with parameter recover_type set to
    RecoverTypeField.DISABLE
    :param lost_key_id: int
    :param auth_hash: str
    :param public_key: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = "{}?hash={}".format(reverse('lost_keys-detail', args=(lost_key_id,)),
                              auth_hash)
    return client.put(url, data={
        'public_key': public_key, 'recover_type': RecoverTypeField.DISABLE})


def retrieve_lost_key_api_call(lost_key_id, auth_hash=None):
    """
    Call to retrieve view
    :param lost_key_id: int
    :param auth_hash: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = "{}?hash={}".format(reverse('lost_keys-detail', args=(lost_key_id,)),
                              auth_hash)
    return client.get(url)



from django.db import models
from django.db.models.manager import Manager
from libs.changes.changes import post_change
from libs.slugify.model import SlugManagerMixin, SlugMixin


class SlugManager(SlugManagerMixin, Manager):
    pass


class Slug(SlugMixin, models.Model):

    objects = SlugManager()

    class Meta:
        db_table = u'vaultier_slug'


def register_signals(model):
    post_change.connect(Slug.objects.on_model, sender=model)



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_403_FORBIDDEN, HTTP_204_NO_CONTENT
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from acls.tests.api import create_role_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from workspaces.tests.api import create_workspace_api_call, \
    list_workspaces_api_call, delete_workspace_api_call, \
    accept_invitation_api_call


class ApiWorkspacePermsTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_000_workspace_anonymous_access(self):

        # user has to be authenticated
        response = create_workspace_api_call(None, name='Workspace')
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        # user has to be authenticated
        response = list_workspaces_api_call(None)
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

    def test_010_workspace_delete_permissions(self):
        # create first user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'tomas@rclick.cz'
        user2 = register_api_call(email=email, nickname='tomas').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace_of_user1').data

        # user1 tries to delete workspace1, should be allowed
        response = delete_workspace_api_call(user1token, workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace_of_user1').data

        # user2 tries to delete workspace1,
        # should not be allowed 404 because of list he does see the workspace
        response = delete_workspace_api_call(user2token, workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        #user1 invites user and grant to user role READ
        user2member = invite_member_api_call(
            user1token, email=user2.get('email'),
            workspace=workspace1.get('id')).data
        user2hash = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2accepted = accept_invitation_api_call(
            user2token, pk=user2member.get('id'), hash=user2hash)
        user2role = create_role_api_call(
            user1token, member=user2member.get('id'),
            to_workspace=workspace1.get('id'), level=RoleLevelField.LEVEL_READ)

        # user2 tries to delete workspace1,
        # should not be allowed 403 because of role only read
        response = delete_workspace_api_call(user2token, workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user2 promotes role of user1 to write
        user2role = create_role_api_call(
            user1token, member=user2member.get('id'),
            to_workspace=workspace1.get('id'),
            level=RoleLevelField.LEVEL_WRITE)

        # user2 tries to delete workspace1, should be allowed,
        # because he has already proper role
        response = delete_workspace_api_call(user2token, workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

    def test_020_workspace_permission_list(self):

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'tomas@rclick.cz'
        register_api_call(email=email, nickname='tomas').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace_of_user1').data

        # create second workspace for user1
        workspace2 = create_workspace_api_call(
            user2token, name='workspace_of_user2').data

        # user1 should see only his workspace
        response = list_workspaces_api_call(user1token)
        self.assertEqual(
            len(response.data),
            1,
            format_response(response)
        )
        self.assertEqual(
            response.data[0].get('name'),
            'workspace_of_user1',
            format_response(response)
        )

        # user2 should see only his workspace
        response = list_workspaces_api_call(user2token)
        self.assertEqual(
            len(response.data),
            1,
            format_response(response)
        )
        self.assertEqual(
            response.data[0].get('name'),
            'workspace_of_user2',
            format_response(response)
        )


def workspace_perms_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        ApiWorkspacePermsTest))
    return suite



import time

from django.core.urlresolvers import reverse
from django.utils import timezone
from rest_framework.test import APIClient

from accounts.business.authentication import Authenticator
from vaultier.test.tools import FileAccessMixin


def auth_api_call(email=None, date=None, signature=None):
    url = reverse('auth-auth')
    client = APIClient()
    m = FileAccessMixin()

    if not date:
        date = timezone.now()

    if not signature:
        privkey = m.read_file('vaultier.key')
        signature = Authenticator.sign(privkey, email, date)

    response = client.post(
        url, {'email': email, 'date': date, 'signature': signature})
    return response


def register_api_call(*args, **kwargs):
    m = FileAccessMixin()
    pubkey = m.read_file('vaultier.pub')
    kwargs['public_key'] = pubkey

    url = reverse('user-list')
    client = APIClient()
    response = client.post(url, kwargs)
    return response


def get_timestamp():
    return int(time.time())



from django.db.models.loading import get_model
from django.db.models.manager import Manager
from acls.business.fields import AclLevelField
from libs.softdelete.softdelete import SoftDeleteManagerMixin


class SecretManager(SoftDeleteManagerMixin, Manager):

    def get_queryset(self):
        queryset = super(SecretManager, self).get_queryset()
        return queryset.filter(
            card__vault__workspace__deleted_at=None,
            card__vault__deleted_at=None,
            card__deleted_at=None,
        )

    def all_for_user(self, user):

        cards = get_model('cards', 'Card').objects.filter(
            acl__user=user,
            acl__level=AclLevelField.LEVEL_READ
        ).distinct()

        secrets = self.filter(
            card__in=cards
        )

        return secrets



from django.core.urlresolvers import reverse
from vaultier.test.tools import VaultierAPIClient


def accept_invitation_api_call(token, pk=None, hash=None):
    url = reverse('invitation-detail', args=(hash,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url)
    return response


def retrieve_invitation_api_call(token, member, hash):
    url = reverse('invitation-detail', args=(hash,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def create_workspace_api_call(token, **kwargs):
    url = reverse('workspace-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_workspace_api_call(token, id):
    url = reverse('workspace-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def retrieve_workspace_api_call(token, pk):
    url = reverse('workspace-detail', args=(pk,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_workspaces_api_call(token):
    url = reverse('workspace-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def update_workspace_api_call(token, pk, **kwargs):
    url = reverse('workspace-detail', args=(pk,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url, kwargs)
    return response


def set_workspace_key_api_call(token, member, key):
    url = reverse('workspace_key-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url, {'workspace_key': key})
    return response


def get_workspace_key_api_call(token, member):
    url = reverse('workspace_key-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_workspace_key_api_call(token):
    url = reverse('workspace_key-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response



from django.db.models.loading import get_model
from libs.version.condition import RequiredFieldEventCondition
from secrets.models import Secret
from libs.version.manipulator import register_manipulator_signal, \
    ModelMovedManipulator
from libs.changes.changes import SOFT_DELETE, INSERT, UPDATE
from libs.version.manipulator import ModelCreatedManipulator, \
    ModelUpdatedManipulator, ModelSoftDeletedManipulator, \
    register_manipulator_class
from versions.models import Version


class SecretManipulatorMixin(object):
    """
    Mixin ensures that secret related is always parent card
    """
    def determine_versioned_related(self):
        return self.version.versioned.get_tree_iterator().get_parent_object()


class SecretCreatedManipulator(SecretManipulatorMixin,
                               ModelCreatedManipulator):
    """
    Mixin ensures that secret related is always parent card
    """
    pass


class SecretUpdatedManipulator(SecretManipulatorMixin,
                               ModelUpdatedManipulator):
    """
    Mixin ensures that secret related is always parent card
    """
    pass


class SecretSoftDeletedManipulator(SecretManipulatorMixin,
                                   ModelSoftDeletedManipulator):
    """
    Mixin ensures that secret related is always parent card
    """
    pass


class SecretMovedManipulator(SecretManipulatorMixin, ModelMovedManipulator):
    """
    Mixin ensures that secret related is always parent card
    Also ensures all secret versions are moved to another card
    """
    def save(self):
        super(SecretMovedManipulator, self).save()

        versions = Version.objects.filter(
            versioned_id=self.version.versioned.id,
            versioned_type__model='secret'
        )

        versions.update(
            versioned_related_id=self.version.versioned_related_id,
        )


class SecretBlobUpdatedManipulator(SecretManipulatorMixin,
                                   ModelUpdatedManipulator):
    """
    Override to ensure that state change stores blob filename
    """
    def store_state(self, revert_data, model):
        revert_data['blob_data'] = revert_data['blob_data'].name
        return super(SecretBlobUpdatedManipulator, self).\
            store_state(revert_data, model)

    def get_diff(self):
        return super(SecretBlobUpdatedManipulator, self).\
            get_diff(fields=['blob_meta'])


def register_signals():
    register_manipulator_class('secret_created_manipulator',
                               SecretCreatedManipulator)
    register_manipulator_class('secret_updated_manipulator',
                               SecretUpdatedManipulator)
    register_manipulator_class('secret_deleted_manipulator',
                               SecretSoftDeletedManipulator)
    register_manipulator_class('secret_moved_manipulator',
                               SecretMovedManipulator)
    register_manipulator_class('secret_blob_updated_manipulator',
                               SecretBlobUpdatedManipulator)

    version_model = get_model('versions', 'Version')

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='secret_deleted_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Secret,
            required_fields=['deleted_at'],
            required_event=SOFT_DELETE,
        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='secret_updated_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Secret,
            required_fields=['name', 'description', 'data'],
            required_event=UPDATE,

        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='secret_moved_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Secret,
            required_fields=['card_id'],
            required_event=UPDATE,

        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='secret_blob_updated_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Secret,
            required_fields=['blob_meta', 'blob_data'],
            required_event=UPDATE,
        )
    )

    register_manipulator_signal(
        version_cls=version_model,
        manipulator_id='secret_created_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Secret,
            required_fields=None,
            required_event=INSERT,
        )
    )



from rest_framework.permissions import BasePermission
from acls.business.fields import AclLevelField
from acls.business.permissions import has_object_acl


class CanManageCardPermission(BasePermission):

    def has_object_permission(self, request, view, obj):

        if view.action in ['retrieve', 'list']:
            return has_object_acl(request.user, obj, AclLevelField.LEVEL_READ)

        if view.action == 'destroy':
            # check permission to card
            return has_object_acl(request.user, obj, AclLevelField.LEVEL_WRITE)

        if view.action in ['create', 'update', 'partial_update']:
            result = True

            # check permission to card
            result = result and has_object_acl(request.user, obj,
                                               AclLevelField.LEVEL_WRITE)

            # check permission to vault
            result = result and has_object_acl(request.user, obj.vault,
                                               AclLevelField.LEVEL_WRITE)

            return result

        return False



from .vault import vault_suite
from .vault_perms import vault_perms_suite
from .vault_softdelete import vault_softdelete_suite
from .vault_version import vault_version_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    s.addTest(vault_suite())
    s.addTest(vault_perms_suite())
    s.addTest(vault_softdelete_suite())
    s.addTest(vault_version_suite())
    return s



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call, update_card_api_call, \
    delete_card_api_call
from libs.version.context import version_context_manager
from libs.version.manipulator import ACTION_CREATED, ACTION_UPDATED, \
    ACTION_SOFTDELETED, ACTION_MOVED
from vaultier.test.tools import AssertionsMixin
from vaults.tests.api import create_vault_api_call
from versions.models import Version
from workspaces.tests.api import create_workspace_api_call


class CardVersionTest(AssertionsMixin, TransactionTestCase):
    def setUp(self):
        version_context_manager.set_enabled(True)

    def create_card(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='workspace').data

        # create vault
        vault = create_vault_api_call(user1token,
                                      workspace=workspace.get('id'),
                                      name='vault'
                                      ).data

        # create card
        card = create_card_api_call(user1token,
                                    name="card_in_vault",
                                    vault=vault.get('id')
                                    ).data

        return user1, user1token, workspace, vault, card

    @unittest.skip("should be fixed asap")
    def test_card_010_create(self):
        user1, user1token, workspace, vault, card = list(self.create_card())

        # check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id')
        )

        # one version should be there
        self.assertEquals(versions.count(), 1)

        # compare version
        self.assert_model(versions[0], {
            'versioned_id': card.get('id'),
            'versioned_related_id': workspace.get('id'),
            'action_id': ACTION_CREATED
        })

        # compare revision data
        self.assert_dict(versions[0].revert_data, {})

        # assert user has been stored with version
        self.assertEqual(versions[0].created_by.id, user1.get('id'))

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id, card.get('id'))

    @unittest.skip("should be fixed asap")
    def test_card_020_update(self):
        user1, user1token, workspace, vault, card = list(self.create_card())

        # update card
        card = update_card_api_call(user1token, card.get('id'),
                                    name='renamed_card',
                                    description='added_card_description'
                                    ).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_UPDATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {
            'name': 'card_in_vault',
            'description': None
        })

        card = update_card_api_call(user1token, card.get('id'),
                                    name='renamed_card_again',
                                    description="changed_card_description"
                                    ).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_UPDATED
        )

        # two versions should be there
        self.assertEquals(versions.count(), 2)

        # compare revision data
        self.assert_dict(versions[1].revert_data, {
            'name': 'renamed_card',
            'description': 'added_card_description'
        })

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id, card.get('id'))

    def test_card_030_delete(self):
        user1, user1token, workspace, vault, card = list(self.create_card())

        # check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_CREATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        delete_card_api_call(user1token, card.get('id'))

        #check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_SOFTDELETED
        )

        # two versions should be there, created and deleted
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {'deleted_at': None})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'vault')
        self.assertEqual(versions[0].versioned_related_id, vault.get('id'))

    def test_card_040_move(self):
        user1, user1token, workspace, vault, card = list(self.create_card())

        # create second vault
        second_vault = create_vault_api_call(
            user1token,
            workspace=workspace.get('id'),
            name='second_vault'
        ).data

        # check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_CREATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        card = update_card_api_call(user1token, card.get('id'),
                                    vault=second_vault.get('id'),
                                    name='renamed_card_again',
                                    description="changed_card_description"
                                    ).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
        )

        # 3 versions should be there, created, update and moved
        self.assertEquals(versions.count(), 3)

        #check version
        versions = Version.objects.filter(
            versioned_type__model='card',
            versioned_id=card.get('id'),
            action_id=ACTION_MOVED
        )

        # compare revision data
        self.assert_dict(versions[0].revert_data,
                         {'vault_id': vault.get('id')})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'card')
        self.assertEqual(versions[0].versioned_related_id, card.get('id'))


def card_version_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(CardVersionTest))
    return suite



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_403_FORBIDDEN, HTTP_200_OK, \
    HTTP_201_CREATED, HTTP_400_BAD_REQUEST
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from acls.models import Role
from acls.tests.api import create_role_api_call, list_role_api_call, \
    update_role_api_call, delete_role_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from vaults.tests.api import create_vault_api_call
from workspaces.tests.api import create_workspace_api_call, \
    accept_invitation_api_call


class ApiRoleTest(TransactionTestCase):
    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_permissions_to_manage_roles(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data
        user1member = Member.objects.all()[0]

        # create second user
        email = 'stepan@rclick.cz'
        register_api_call(email=email, nickname='stepan').data
        user2token = auth_api_call(email=email).data.get('token')

        # user2 tries to add role to user1 and workspace1 - should be forbidden
        response = create_role_api_call(user2token, user1member.id,
                                        level=RoleLevelField.LEVEL_READ,
                                        to_workspace=workspace1.get('id'))
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

    def test_020_permissions_to_see_list_of_roles(self):
        # list roles as anonymous should be forbidden
        response = list_role_api_call(None)
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data
        user1member = Member.objects.all()[0]

        # user1 list roles, should see one role
        response = list_role_api_call(user1token)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))
        self.assertEqual(len(response.data), 1, format_response(response))

        # create another user
        email = 'stepan@rclick.cz'
        register_api_call(email=email, nickname='stepan').data
        user2token = auth_api_call(email=email).data.get('token')

        # user2 list roles, should see nothing, event there is one role in db
        response = list_role_api_call(user2token)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))
        self.assertEqual(len(response.data), 0, format_response(response))

        # user1 invites user 2, user 2 should see 2 role
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        accept_invitation_api_call(user2token, pk=user2member.get('id'),
                                   hash=user2invitation)
        create_role_api_call(user1token,
                             user2member.get('id'),
                             workspace1.get('id'),
                             level=RoleLevelField.LEVEL_WRITE)
        response = list_role_api_call(user2token)

        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))
        self.assertEqual(len(response.data), 2, format_response(response))

        # user1 changes role level to only read, user2 should
        # be able to list roles
        create_role_api_call(user1token,
                             user2member.get('id'),
                             workspace1.get('id'),
                             level=RoleLevelField.LEVEL_READ)

        response = list_role_api_call(user2token)

        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))
        self.assertEqual(len(response.data), 2, format_response(response))

        # create another user
        email = 'michal@rclick.cz'
        register_api_call(email=email, nickname='stepan').data
        user3token = auth_api_call(email=email).data.get('token')

        # user3 should not be able to list roles
        response = list_role_api_call(user3token)

        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )
        self.assertEqual(
            len(response.data),
            0,
            format_response(response)
        )

    def test_030_role_is_saved_only_once(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace1').data

        # user1 invites user2 to workspace1
        user2member = invite_member_api_call(
            user1token, 'jakub@rclick.cz', workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash

        # user1 grant read role to user2
        response = create_role_api_call(user1token, user2member.get('id'),
                                        level=RoleLevelField.LEVEL_WRITE,
                                        to_workspace=workspace1.get('id')
                                        )
        self.assertEqual(
            response.status_code,
            HTTP_201_CREATED,
            format_response(response)
        )

        # user1 grant read role to user2
        response = create_role_api_call(user1token,
                                        user2member.get('id'),
                                        level=RoleLevelField.LEVEL_READ,
                                        to_workspace=workspace1.get('id')
                                        )
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         format_response(response))

        # check roles in db. There should be one role with read
        # permission for user2
        roles = Role.objects.filter(member=user2member.get('id'),
                                    to_workspace=workspace1.get('id'))
        self.assertEqual(roles.count(), 1, format_response(response))

        # role should have level read as it was rewriten
        self.assertEqual(roles[0].level, RoleLevelField.LEVEL_READ,
                         format_response(response))

    def test_040_role_has_at_least_one_associated_object(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data

        # user1 invites user2 to workspace1
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash

        # user1 grant read role to user2
        response = create_role_api_call(user1token,
                                        user2member.get('id'),
                                        level=RoleLevelField.LEVEL_WRITE)
        self.assertEqual(response.status_code,
                         HTTP_400_BAD_REQUEST,
                         format_response(response))

    def test_050_create_role(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create another user
        email = 'stepan@rclick.cz'
        register_api_call(email=email, nickname='stepan').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token,
                                       name='vault1',
                                       workspace=workspace1.get('id')).data

        # user1 invites user 2
        user2member = invite_member_api_call(user1token,
                                             'jakub@rclick.cz',
                                             workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        accept_invitation_api_call(user2token, pk=user2member.get('id'),
                                   hash=user2invitation)

        # user 1 creates role for user 2
        response = create_role_api_call(user1token, user2member.get('id'),
                                        to_vault=vault1.get('id'),
                                        level=RoleLevelField.LEVEL_READ)
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         format_response(response))

        # user 2 tries to create role, should be forbidden
        response = create_role_api_call(user2token,
                                        user2member.get('id'),
                                        to_vault=vault1.get('id'),
                                        level=RoleLevelField.LEVEL_READ)
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

    def test_050_edit_and_delete_role(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create another user
        email = 'stepan@rclick.cz'
        register_api_call(email=email, nickname='stepan').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault1',
                                       workspace=workspace1.get('id')).data

        # user1 invites user 2
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        accept_invitation_api_call(user2token, pk=user2member.get('id'),
                                   hash=user2invitation)

        # user 1 creates role for user 2
        role = create_role_api_call(user1token, user2member.get('id'),
                                    to_vault=vault1.get('id'),
                                    level=RoleLevelField.LEVEL_READ
                                    ).data

        # user 2 tries to update role, should be forbidden
        response = update_role_api_call(user2token, role.get('id'),
                                        RoleLevelField.LEVEL_WRITE)
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        #user 2  delete role, should be forbidden
        response = delete_role_api_call(user2token, role.get('id'))
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        #user 1 update role to write,
        response = update_role_api_call(user1token, role.get('id'),
                                        RoleLevelField.LEVEL_WRITE)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))

        #user 2 tries to update role, should be allowed
        response = update_role_api_call(user2token, role.get('id'),
                                        RoleLevelField.LEVEL_WRITE)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))


def role_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiRoleTest))
    return suite



from rest_framework import serializers
from accounts.serializers import RelatedUserSerializer
from secrets.business.fields import SecretPermsField, BlobDataField
from secrets.models import Secret


class SecretSerializer(serializers.ModelSerializer):
    created_by = RelatedUserSerializer(read_only=True)
    perms = SecretPermsField()
    blob_meta = serializers.CharField(read_only=True)

    def restore_fields(self, data, files):
        if self.context.get('view').action == 'update' or \
                self.context.get('view').action == 'partial_update':
            self.fields.get('type').read_only = True
        return super(SecretSerializer, self).restore_fields(data, files)

    class Meta:
        model = Secret
        fields = ('id', 'type', 'name', 'data', 'blob_meta', 'card', 'perms',
                  'created_at', 'updated_at', 'created_by')


class SecretBlobSerializer(serializers.ModelSerializer):
    created_by = RelatedUserSerializer(read_only=True)
    blob_data = BlobDataField()

    class Meta:
        model = Secret
        fields = ('id', 'blob_meta', 'blob_data')



##################################################
#Perms signals registration
##################################################
from acls.business.perms.signals import register_signals as \
    perm_register_signals
perm_register_signals()

##################################################
#Slugs signals registration
##################################################
from slugs.models import Slug
from workspaces.models import Workspace
from vaults.models import Vault
from cards.models import Card
from libs.slugify.model import register_signals as slug_register_signals
slug_register_signals(Slug, Workspace)
slug_register_signals(Slug, Vault)
slug_register_signals(Slug, Card)

##################################################
# Versioning signals
##################################################
from workspaces.business.version import register_signals as \
    workspace_register_version_signals
from vaults.business.version import register_signals as \
    vault_register_version_signals
from cards.business.version import register_signals as \
    card_register_version_signals
from secrets.business.version import register_signals as \
    secret_register_version_signals

workspace_register_version_signals()
vault_register_version_signals()
card_register_version_signals()
secret_register_version_signals()

##################################################
#Role custom signals
##################################################
from acls.models import register_signals as role_register_signals
role_register_signals()

##################################################
#LostKey custom signals
##################################################
from accounts.models import register_signals as lostkey_register_signals
lostkey_register_signals()



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_201_CREATED, HTTP_200_OK, \
    HTTP_204_NO_CONTENT
from accounts.tests.api import auth_api_call, register_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from vaults.tests.api import create_vault_api_call, delete_vault_api_call, \
    list_vaults_api_call, retrieve_vault_api_call, update_vault_api_call
from workspaces.tests.api import create_workspace_api_call


class ApiVaultTest(TransactionTestCase):

    def create_vault(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='Workspace').data

        #create vault
        response = create_vault_api_call(
            user1token, name="vault_in_workspace",
            workspace=workspace.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_201_CREATED,
            format_response(response)
        )
        vault = response.data

        return user1, user1token, workspace, vault

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_create_vault(self):
        self.create_vault()

    def test_020_delete_vault(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        #delete vault
        response = delete_vault_api_call(user1token, vault.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

    def test_030_list_vault(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        #list vaults
        response = list_vaults_api_call(
            user1token, workspace=workspace.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

    def test_040_retrieve_vault(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        #list vaults
        response = retrieve_vault_api_call(user1token, vault.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_050_move_vault_should_not_be_allowed(self):
        user1, user1token, workspace, vault = list(self.create_vault())
        response = update_vault_api_call(
            user1token, vault.get('id'), workspace=2, name="changed_name")
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # workspace should be unchanged
        self.assertEqual(
            response.data.get('workspace'),
            1,
            format_response(response)
        )


def vault_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiVaultTest))
    return suite



from django.core.urlresolvers import reverse
from vaultier.test.tools import VaultierAPIClient


def create_card_api_call(token, **kwargs):
    url = reverse('card-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_card_api_call(token, id):
    url = reverse('card-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def update_card_api_call(token, pk, **kwargs):
    url = reverse('card-detail', args=(pk,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.patch(url, data=kwargs)
    return response


def retrieve_card_api_call(token, pk):
    url = reverse('card-detail', args=(pk,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_cards_api_call(token, vault=None):
    url = reverse('card-list')
    client = VaultierAPIClient()
    client.token(token)

    data = {}
    if vault:
        data['vault'] = vault

    response = client.get(url, **data)
    return response
